@@ -1,9 +1,9 @@
 import os
 import re
-import signal
 import subprocess
 import sys
 import threading
+from time import sleep
 
 os_win = os.name == "nt"
 
@@ -26,6 +26,7 @@ class Process:
     shell = True
     hidden = True
     exception = None
+    kill_children = True
 
     def __init__(self, command, timeout=3600, block=True, encoding=None, popen_override=None, shell=True, hidden=True):
         self.command = command
@@ -49,7 +50,7 @@ class Process:
                 self.thread.join(1)
         else:
             while self.process is None and self.exception is None:
-                pass
+                sleep(0.010)
 
         if self.exception:
             raise self.exception
@@ -150,21 +151,24 @@ class Process:
         if not self.process:
             return
 
-        if self.shell:
-            if os_win:
-                subprocess.check_output(
-                    "taskkill /t /f /pid " + str(self.process.pid), startupinfo=hidden_startupinfo()
-                )
-            else:
-                subprocess.check_output(
-                    "pkill -P %s" % self.process.pid, shell=True
-                )
+        if self.kill_children:
+            self.kill_tree()
         else:
             self.process.kill()
 
         if block:
             self.thread.join()
 
+    def kill_tree(self):
+        if os_win:
+            subprocess.check_output(
+                "taskkill /t /f /pid " + str(self.process.pid), startupinfo=hidden_startupinfo()
+            )
+        else:
+            subprocess.check_output(
+                "pkill -P %s" % self.process.pid, shell=True
+            )
+
 
 _detected_encoding = None
 
@@ -199,10 +203,9 @@ def execute_command(command, timeout=60, valid_codes=0, encoding=None, popen_ove
         valid_codes = [valid_codes]
 
     if process.return_code() not in valid_codes:
-        raise ProcessException(
-            "command " + command + " failed with code " + str(process.return_code()) +
-            ", output: " + process.read()
-        )
+        raise ProcessException("command %s failed with code %s, output: %s" % (
+            command, process.return_code(), process.read()
+        ))
 
     return process
 
